#include <io.h>
#include <interrupt.h>
#include "signal.h"
#include "uart.h"

/* UART global variables */               
volatile U8 echoSerial;  // maybe used someday!
volatile U8 UCRstate;    // state of UCR register before killing UCR interrupts


/* end-of-line string = 'Line End' + 'Line Feed' character */
unsigned char UART_pszEndOfLine[3] = {0x0d,0x0a,0};


////////////////////////////////////////////////////////////////////
// gccAVR - polled 

void UART_Init()
{
    // echo chars input
    //echoSerial        = 1;

    // Enable External SRAM
    //outp(BV(SRE) | BV(SRW), MCUCR);
    outp(BV(SRE), MCUCR);  // no wait states 

    // transmit and receive on, transmit and receive interrupts OFF
    outp(BV(RXEN) | BV(TXEN), UCR); 

    /* set 9600 baud rate for 4mhz processor*/
    outp(UART_BAUD_SELECT, UBRR);  
}

//
// xmt char (no interrupts used)
//
void UART_SendByte(U8 Data)
{       
    /* wait for UART to become available */
    while (!(inp(USR) & (1<<UDRE)));
    
    /* send byte to UART data buffer */
    outp( Data, UDR );
}

//
// rcv char (no interrupts used)
//
U8 UART_ReceiveByte()
{
        
    /* wait for UART to indicate that a character has been received */
    while ((inp(USR) & (1<<RXC)) == 0);

    /* read byte from UART data buffer */
    return inp(UDR);
}

//
// print a string (no interrupts used)
//
void UART_PrintfStr(char* pBuf)
{
   unsigned char i;
   
   UCRstate = inp(UCR);                // save uart state
   outp(BV(RXEN) | BV(TXEN), UCR);     // uart intr off

   /* Send string character */
   for (i=0; i<78; ++i)
   {
      if (pBuf[i] == 0)
         break;
      UART_SendByte((U8)pBuf[i]);
   }

   outp(UCRstate, UCR);                // restore uart state
}

//
// print end of line (no interrupts used)
//
void UART_PrintfEndOfLine()
{
   UCRstate = inp(UCR);                // save uart state
   outp(BV(RXEN) | BV(TXEN), UCR);     // uart intr off
    
   UART_PrintfStr((S8*)UART_pszEndOfLine);

   outp(UCRstate, UCR);                // restore uart state
}

//
// print a hex digit (no interrupts used)
//
void UART_PrintfU4(U8 Data)
{
    /* Send 4-bit hex value */
    U8 Character = Data&0x0f;
    if (Character>9)
    {
        Character+='A'-10;
    }
    else
    {
        Character+='0';
    }

    UART_SendByte(Character);
}

//
// 'sprint' a hex digit to users buffer
//
void UART_sPrintfU4(U8 Data, U8* Str)
{
   /* save 4-bit hex value */
   U8 Character = Data&0x0f;
   if (Character>9)
   {
       Character+='A'-10;
   }
   else
   {
       Character+='0';
   }
   *Str = Character;
   *(Str+1) = 0;
}

//
// print a byte in hex (no interrupts used)
//
void UART_PrintfU8(U8 Data)
{
   UCRstate = inp(UCR);                // save uart state   
   outp(BV(RXEN) | BV(TXEN), UCR);     // uart intr off

   /* Send 8-bit hex value */
   UART_PrintfU4(Data>>4);
   UART_PrintfU4(Data   );

   outp(UCRstate, UCR);                // restore uart state
}

//
// 'sprint' a byte in hex to users buffer
//
void UART_sPrintfU8(U8 Data, U8* Str)
{
    /* Save 8-bit hex value */
    UART_sPrintfU4(Data>>4, Str);
    UART_sPrintfU4(Data   , Str+1);
}

//
// print a word in hex (no interrupts used)
//
void UART_PrintfU16(U16 Data)
{
   UCRstate = inp(UCR);                // save uart state   
   outp(BV(RXEN) | BV(TXEN), UCR);     // uart intr off

   /* Send 16-bit hex value */
   UART_PrintfU4(Data>>12);
   UART_PrintfU4(Data>> 8);
   UART_PrintfU4(Data>> 4);
   UART_PrintfU4(Data    );

   outp(UCRstate, UCR);                // restore uart state
}

//
// 'sprint' a word in hex to users buffer
//
void UART_sPrintfU16(U16 Data, U8* Str)
{
    /* Save 16-bit hex value */
    UART_sPrintfU4(Data>>12, Str);
    UART_sPrintfU4(Data>> 8, Str+1);
    UART_sPrintfU4(Data>> 4, Str+2);
    UART_sPrintfU4(Data    , Str+3);
}
//
// print a long in hex (no interrupts used)
//

void UART_PrintfU32(U32 Data)
{

   UCRstate = inp(UCR);                // save uart state   
   outp(BV(RXEN) | BV(TXEN), UCR);     // uart intr off

   /* Send 32-bit hex value */
   UART_PrintfU4(Data>>28);
   UART_PrintfU4(Data>>24);
   UART_PrintfU4(Data>>20);
   UART_PrintfU4(Data>>16);
   UART_PrintfU4(Data>>12);
   UART_PrintfU4(Data>> 8);
   UART_PrintfU4(Data>> 4);
   UART_PrintfU4(Data    );

   outp(UCRstate, UCR);                // restore uart state
}

//
// 'sprint' a long in hex to users buffer
//
void UART_sPrintfU32(U32 Data, U8* Str)
{
    /* Send 32-bit hex value */
    UART_sPrintfU4(Data>>28, Str);
    UART_sPrintfU4(Data>>24, Str+1);
    UART_sPrintfU4(Data>>20, Str+2);
    UART_sPrintfU4(Data>>16, Str+3);
    UART_sPrintfU4(Data>>12, Str+4);
    UART_sPrintfU4(Data>> 8, Str+5);
    UART_sPrintfU4(Data>> 4, Str+6);
    UART_sPrintfU4(Data    , Str+7);
}
